Trait scuttlebutt::SemiHonest
source · pub trait SemiHonest { }Expand description
A marker trait denoting that the given scheme is semi-honest secure.
pub trait SemiHonest { }A marker trait denoting that the given scheme is semi-honest secure.